@@ -1,10 +1,5 @@
 {-# LANGUAGE CPP #-}
 
-#if MIN_VERSION_base(4,6,0)
--- Control.Concurrent.QSem is deprecated in base-4.6.0.*
-{-# OPTIONS_GHC -fno-warn-deprecations #-}
-#endif
-
 -- |
 -- Stability: provisional
 module Test.Hspec.Core.Runner (
@@ -30,24 +25,20 @@ module Test.Hspec.Core.Runner (
 import           Prelude ()
 import           Test.Hspec.Core.Compat
 
-import           Control.Monad
 import           Data.Maybe
 import           System.IO
 import           System.Environment (getProgName, getArgs, withArgs)
 import           System.Exit
 import qualified Control.Exception as E
-import           Control.Concurrent
 
 import           System.Console.ANSI (hHideCursor, hShowCursor)
 import qualified Test.QuickCheck as QC
-import           Control.Monad.IO.Class (liftIO)
 
 import           Test.Hspec.Core.Util (Path)
 import           Test.Hspec.Core.Spec
 import           Test.Hspec.Core.Config
 import           Test.Hspec.Core.Formatters
 import           Test.Hspec.Core.Formatters.Internal
-import qualified Test.Hspec.Core.Formatters.Internal as Formatter
 import           Test.Hspec.Core.FailureReport
 import           Test.Hspec.Core.QuickCheckUtil
 
@@ -83,7 +74,7 @@ applyDryRun c
   | otherwise = id
   where
     markSuccess :: Item () -> Item ()
-    markSuccess item = item {itemExample = safeEvaluateExample Success}
+    markSuccess item = item {itemExample = safeEvaluateExample (Result "" Success)}
 
     removeCleanup :: SpecTree () -> SpecTree ()
     removeCleanup spec = case spec of
@@ -160,25 +151,43 @@ runSpec config spec = do
         seed = (fromJust . configQuickCheckSeed) config
         qcArgs = configQuickCheckArgs config
 
-    jobsSem <- newQSem =<< case configConcurrentJobs config of
+    concurrentJobs <- case configConcurrentJobs config of
       Nothing -> getDefaultConcurrentJobs
-      Just maxJobs -> return maxJobs
+      Just n -> return n
 
     useColor <- doesUseColor h config
 
-    filteredSpec <- filterSpecs config . applyDryRun config <$> runSpecM spec
+    let params = Params (configQuickCheckArgs config) (configSmallCheckDepth config)
 
-    withHiddenCursor useColor h $
-      runFormatM useColor (configDiff config) (configHtmlOutput config) (configPrintCpuTime config) seed h $ do
-        runFormatter jobsSem useColor h config formatter filteredSpec `finally_` do
-          Formatter.interpret $ failedFormatter formatter
+    filteredSpec <- map (toEvalTree params) . filterSpecs config . applyDryRun config <$> runSpecM spec
 
-        Formatter.interpret $ footerFormatter formatter
+    (total, failures) <- withHiddenCursor useColor h $ do
+      let
+        formatConfig = FormatConfig {
+          formatConfigHandle = h
+        , formatConfigUseColor = useColor
+        , formatConfigUseDiff = configDiff config
+        , formatConfigHtmlOutput = configHtmlOutput config
+        , formatConfigPrintCpuTime = configPrintCpuTime config
+        , formatConfigUsedSeed =  seed
+        }
+        evalConfig = EvalConfig {
+          evalConfigFormat = formatterToFormat formatter formatConfig
+        , evalConfigConcurrentJobs = concurrentJobs
+        , evalConfigFastFail = configFastFail config
+        }
+      runFormatter evalConfig filteredSpec
 
-        xs <- map failureRecordPath <$> Formatter.interpret getFailMessages
-        liftIO $ dumpFailureReport config seed qcArgs xs
+    dumpFailureReport config seed qcArgs failures
+    return (Summary total (length failures))
 
-        Summary <$> Formatter.interpret getTotalCount <*> Formatter.interpret getFailCount
+toEvalTree :: Params -> SpecTree () -> EvalTree
+toEvalTree params = go
+  where
+    go t = case t of
+      Node s xs -> Node s (map go xs)
+      NodeWithCleanup c xs -> NodeWithCleanup (c ()) (map go xs)
+      Leaf (Item requirement loc isParallelizable e)  -> Leaf (EvalItem requirement loc (fromMaybe False isParallelizable) (e params $ ($ ())))
 
 dumpFailureReport :: Config -> Integer -> QC.Args -> [Path] -> IO ()
 dumpFailureReport config seed qcArgs xs = do
