@@ -19,6 +19,7 @@ module Test.Hspec.Core.Spec (
 , xdescribe
 , xcontext
 , parallel
+, sequential
 
 -- * The @SpecM@ monad
 , module Test.Hspec.Core.Spec.Monad
@@ -30,6 +31,9 @@ module Test.Hspec.Core.Spec (
 , module Test.Hspec.Core.Tree
 ) where
 
+import           Prelude ()
+import           Test.Hspec.Core.Compat
+
 import qualified Control.Exception as E
 import           Data.CallStack
 
@@ -41,22 +45,22 @@ import           Test.Hspec.Core.Tree
 import           Test.Hspec.Core.Spec.Monad
 
 -- | The @describe@ function combines a list of specs into a larger spec.
-describe :: String -> SpecWith a -> SpecWith a
+describe :: HasCallStack => String -> SpecWith a -> SpecWith a
 describe label spec = runIO (runSpecM spec) >>= fromSpecList . return . specGroup label
 
 -- | @context@ is an alias for `describe`.
-context :: String -> SpecWith a -> SpecWith a
+context :: HasCallStack => String -> SpecWith a -> SpecWith a
 context = describe
 
 -- |
 -- Changing `describe` to `xdescribe` marks all spec items of the corresponding subtree as pending.
 --
 -- This can be used to temporarily disable spec items.
-xdescribe :: String -> SpecWith a -> SpecWith a
-xdescribe label spec = before_ pending $ describe label spec
+xdescribe :: HasCallStack => String -> SpecWith a -> SpecWith a
+xdescribe label spec = before_ pending_ $ describe label spec
 
 -- | @xcontext@ is an alias for `xdescribe`.
-xcontext :: String -> SpecWith a -> SpecWith a
+xcontext :: HasCallStack => String -> SpecWith a -> SpecWith a
 xcontext = xdescribe
 
 -- | The @it@ function creates a spec item.
@@ -82,7 +86,7 @@ specify = it
 --
 -- This can be used to temporarily disable a spec item.
 xit :: (HasCallStack, Example a) => String -> a -> SpecWith (Arg a)
-xit label action = before_ pending $ it label action
+xit label action = before_ pending_ $ it label action
 
 -- | @xspecify@ is an alias for `xit`.
 xspecify :: (HasCallStack, Example a) => String -> a -> SpecWith (Arg a)
@@ -91,7 +95,14 @@ xspecify = xit
 -- | `parallel` marks all spec items of the given spec to be safe for parallel
 -- evaluation.
 parallel :: SpecWith a -> SpecWith a
-parallel = mapSpecItem_ $ \item -> item {itemIsParallelizable = True}
+parallel = mapSpecItem_ (setParallelizable True)
+
+-- | `sequential` marks all spec items of the given spec to be evaluated sequentially.
+sequential :: SpecWith a -> SpecWith a
+sequential = mapSpecItem_ (setParallelizable False)
+
+setParallelizable :: Bool -> Item a -> Item a
+setParallelizable value item = item {itemIsParallelizable = itemIsParallelizable item <|> Just value}
 
 -- | `pending` can be used to mark a spec item as pending.
 --
@@ -101,11 +112,14 @@ parallel = mapSpecItem_ $ \item -> item {itemIsParallelizable = True}
 -- > describe "fancyFormatter" $ do
 -- >   it "can format text in a way that everyone likes" $
 -- >     pending
-pending :: Expectation
-pending = E.throwIO (Pending Nothing)
+pending :: HasCallStack => Expectation
+pending = E.throwIO (Pending location Nothing)
+
+pending_ :: Expectation
+pending_ = (E.throwIO (Pending Nothing Nothing))
 
 -- |
 -- `pendingWith` is similar to `pending`, but it takes an additional string
 -- argument that can be used to specify the reason for why the spec item is pending.
-pendingWith :: String -> Expectation
-pendingWith = E.throwIO . Pending . Just
+pendingWith :: HasCallStack => String -> Expectation
+pendingWith = E.throwIO . Pending location . Just
